<?php

class ProductEntry {

    private $shortName; // the name that shall appear on the button of the waiter
    private $priceA; // the price of the product (default)
    private $priceB; // price level B
    private $priceC; // price level C
    private $barcode;
    private $tax;
    private $togotax;
    private $taxaustria;
    private $amount;
    private $longName; // the name that shall appear on the receipt
    private $prodid = null;
    private $available = 1;
    private $favorite = "";
    private $audio = "";
    private $category = null;
    private $unit = null;
    private $days = null;
    private $prodimageid = 0;
    private $display = 'KG';
    
    private static $PRICE_B = "PreisB";
    private static $PRICE_C = "PreisC";
    private static $BARCODE = "Barcode";
    private static $TAX = "Normal-Steuersatz";
    private static $TOGOTAX = "To-Go-Steuersatz";
    private static $TAXAUSTRIA = "Steuersatz-Austria";
    private static $AMOUNT = "Menge";
    private static $SHORTNAME = "Kurzname";
    private static $AVAILABLE = "vorhanden";
    private static $UNIT = "Einheit";
    private static $DAYS = "Tage";
    private static $ID = "ID";
    private static $IMAGEID = "Bildnr";
    private static $DISPLAY = "Anzeige";

    
    public function createWithSubsetOfData($prodtypeid,$longname,$shortname,$priceA,$priceB,$priceC) {
	    $this->longName = $longname;
	    $this->shortName = $shortname;
	    $this->priceA = $priceA;
	    $this->priceB = $priceB;
	    $this->priceC = $priceC;
	    $this->category = $prodtypeid;
	    $this->available = 1;
	    $this->favorite = 0;
	    $this->unit = null;
	    $this->tax = 1;
	    $this->togotax = 2;
	    $this->taxaustria = null;
	    $this->display = null;
	    $this->days = '0123456';
	    $this->audio = null;
	    $this->barcode = "";
	    $this->amount = null;
    }
    
    public function createFromPostData($postData) {
	$prodCols = DbUtils::$prodCols;
	foreach($prodCols as $aCol) {
	    if (isset($aCol["property"])) {
		$memberVar = $aCol["property"];
		if (isset($postData[$aCol["col"]])) {
		    $this->$memberVar = $postData[$aCol["col"]];
		}
	    }
	}
	
	if (!is_numeric($this->available)) {
	    $this->available = 1;
	}
	if (!is_numeric($this->favorite)) {
	    $this->favorite = 0;
	}
	if (!is_numeric($this->priceA)) {
	    $this->priceA = 0.00;
	}
	if (!is_numeric($this->priceB)) {
	    $this->priceB = 0.00;
	}
	if (!is_numeric($this->priceC)) {
	    $this->priceC = 0.00;
	}
	if ($this->unit == 'null') {
		$this->unit = null;
	}
	if ($this->taxaustria == 'null') {
		$this->taxaustria = null;
	}
	if (!is_numeric($this->unit)) {
		$this->unit = null;
	} else {
		if ($this->unit == 0) {
			$this->unit = null;
		}
	}
	if (($this->display != 'K') && ($this->display != 'G')) {
		$this->display = null;
	}
	
	if (is_null($this->days) || ($this->days == "")) {
		$this->days = '0123456';
	}
	if (($this->tax == "null") || ($this->tax == 0)) {
	    $this->tax = 1;
	}
	if (($this->togotax == "null") || ($this->togotax == 0)) {
	    $this->togotax = 2;
	}
	if ($this->audio == '') {
	    $this->audio = null;
	}
	if (trim($this->shortName) == "") {
	    $this->shortName = $this->longName;
	}
	if (is_null($this->barcode) || (trim($this->barcode) == "")) {
		$this->barcode = "";
	}
	if ($this->amount == "null") {
	    $this->amount = null;
	}
    }
    
    public function createProductInDb($pdo) {
	$sql = "INSERT INTO `%products%` (`id`,`shortname`,`longname`,`priceA`,`priceB`,`priceC`,`barcode`,`unit`,`display`,`days`,`tax`,`togotax`,`taxaustria`,`amount`,`category`,`favorite`,`available`,`audio`) VALUES (NULL,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
	$stmt = $pdo->prepare(DbUtils::substTableAlias($sql));
	if ($this->days == '0123456') {
		$this->days = null;
	}
	$barcode = $this->barcode;
	if (strlen($barcode) > 25) {
		$barcode = substr($barcode, 0, 25);
	}
	$stmt->execute(array($this->shortName,$this->longName,$this->priceA,$this->priceB,$this->priceC,$barcode,$this->unit,$this->display,$this->days,$this->tax,$this->togotax,$this->taxaustria,$this->amount,$this->category,$this->favorite,$this->available,$this->audio));

	return ($pdo->lastInsertId());
    }
    
    public function applyProductInDb($pdo) {
	$updateSql = "UPDATE %products% SET shortname=?, longname=?, priceA=?, priceB=?, priceC=?, barcode=?, unit=?, display=?, days=?, tax=?, togotax=?, taxaustria=?, amount=?,available=?, audio=?, favorite=? WHERE id=?";		
	$stmt = $pdo->prepare(DbUtils::substTableAlias($updateSql));
	if ($this->days == '0123456') {
		$this->days = null;
	}
	$stmt->execute(array($this->shortName,$this->longName,$this->priceA,$this->priceB,$this->priceC,$this->barcode,$this->unit,$this->display,$this->days,$this->tax,$this->togotax,$this->taxaustria,$this->amount,$this->available,$this->audio,$this->favorite,$this->prodid));
    }
    
    // 
    public static function createProductStr($aProd, $decpoint) {

	$shortname = $aProd['shortname'];
	$longname = $aProd['longname'];
	$prodId = $aProd['id'];

	$available = $aProd['available'];
	$priceA = str_replace('.', $decpoint, $aProd['priceA']);
	$priceB = str_replace('.', $decpoint, $aProd['priceB']);
	$priceC = str_replace('.', $decpoint, $aProd['priceC']);
	$barcode = $aProd['barcode'];
	$unit = $aProd['unit'];
	$days = $aProd['days'];
	$prodimageid = $aProd['prodimageid'];
	$display = $aProd['display'];
	$tax = str_replace('.', $decpoint, $aProd['tax']);
	$togotax = str_replace('.', $decpoint, $aProd['togotax']);
	$taxaustria = $aProd['taxaustria'];
	$amount = $aProd['amount'];

	$prodText = "$longname; $priceA";
	$extArr = array();

	if ($shortname != $longname) {
	    $extArr[] = self::$SHORTNAME . ":" . $shortname;
	}

	if (!is_null($tax) && ($tax != "null") && ($tax != "1")) {
	    $extArr[] = self::$TAX . ":" . $tax;
	}
	if (!is_null($togotax) && ($togotax != "null") && ($togotax != "2")) {
	    $extArr[] = self::$TOGOTAX . ":" . $togotax;
	}

	if (($priceB != $priceA) || ($priceC != $priceA)) {
	    $extArr[] = self::$PRICE_B . ":$priceB";
	    $extArr[] = self::$PRICE_C . ":$priceC";
	}
	
	if (($barcode != "") && ($barcode != 'null')) {
		$extArr[] = self::$BARCODE . ":$barcode";
	}
	
	if (!is_null($taxaustria) && ($taxaustria != 'null')) {
	    $extArr[] = self::$TAXAUSTRIA . ":$taxaustria";
	}
	
	if (!is_null($amount) && ($amount != 'null')) {
	    $extArr[] = self::$AMOUNT . ":$amount";
	}

	if ($available == 0) {
	    $extArr[] = self::$AVAILABLE . ":nein";
	}
	
	if (!is_null($unit)) {
		foreach(CommonUtils::$g_units_arr as $u) {
			if ($u["value"] == $unit) {
				$extArr[] = self::$UNIT . ":" . $u["text"];
			}
		}
	}
	
	if (!is_null($days)) {
		$extArr[] = self::$DAYS . ":$days";
	}
	
	if (!is_null($prodId)) {
	    $extArr[] = self::$ID . ":" . $prodId;
	}

	if (!is_null($prodimageid) && ($prodimageid != 0)) {
		$extArr[] = self::$IMAGEID . ":" . $prodimageid;
	}
	
	if (!is_null($display) && (strtoupper($display) != "KG")) {
		$display = strtoupper($display);
		if (($display == 'K') || ($display == 'G')) {
			$extArr[] = self::$DISPLAY . ":" . $display;
		}
	}
	
	if (count($extArr) > 0) {
	    $prodText .= " # " . join("; ", $extArr);
	}

	return $prodText;
    }

    function parse($aTextLine) {
	try {
	    $aTextLine = trim($aTextLine);
	    $propertyparts = explode('#', $aTextLine, 2);
	    
	    $shortAndPriceA = $propertyparts[0];
	    $basic = explode(';',$shortAndPriceA);
	    $this->longName = $basic[0];
	    $this->priceA = floatval(str_replace(",",".",(string) $basic[1]));
	    
	    $this->priceB = null;
	    $this->priceC = null;
	    $this->barcode = null;
	    $this->unit = null;
	    $this->days = null;
	    $this->prodimageid = null;
	    $this->display = null;
	    $this->shortName = null;
	    
	    $this->tax = 1;
	    $this->togotax = 2;
	    $this->taxaustria = 0;
	    $this->amount = "";
	    
	    if ((count($propertyparts) > 1) && (trim($propertyparts[1]) != "")) {
		$exts = explode(";",$propertyparts[1]);
		foreach($exts as $anExtProp) {
                        if (trim($anExtProp) == "") {
                                continue;
                        }
		    $parts = explode(":",$anExtProp);
		    $identifier = trim($parts[0]);
                    if (count($parts) == 1) {
                            continue;
                    }
		    $value = trim($parts[1]);
		    if ($identifier == self::$PRICE_B) {
			$this->priceB = floatval(str_replace(",",".",(string) $value));
		    } else if ($identifier == self::$PRICE_C) {
			$this->priceC = floatval(str_replace(",",".",(string) $value));
		    } else if ($identifier == self::$AVAILABLE) {
			if (($value == "ja") || ($value == "yes") || ($value == "si") || ($value == "1")) {
			    $this->available = 1;
			} else {
			    $this->available = 0;
			}
		    } else if ($identifier == self::$UNIT) {
			    if ($value == "Stück") {
				    $this->unit = null;
			    } else {
				    $vlower = strtolower($value);
				    foreach(CommonUtils::$g_units_arr as $u) {
					    if (strtolower($u["text"]) == $vlower) {
						    $this->unit = $u["value"];
						    break;
					    }
				    }
			    }
		    } else if ($identifier == self::$DAYS) {
			    $this->days = $value;
		    } else if ($identifier == self::$TAX) {
			$this->tax = intval(str_replace(",",".",(string) $value));
		    } else if ($identifier == self::$TOGOTAX) {
			$this->togotax = intval(str_replace(",",".",(string) $value));
		    } else if ($identifier == self::$SHORTNAME) {
			$this->shortName = $value;
		    } else if ($identifier == self::$BARCODE) {
			    $this->barcode = $value;
		    } else if ($identifier == self::$ID) {
			$this->prodid = $value;
		    } else if ($identifier == self::$TAXAUSTRIA) {
			if (($value<0) || ($value > 4)) {
			    $value = null;
			}
			$this->taxaustria = $value;
		    } else if ($identifier == self::$AMOUNT) {
			$this->amount = $value;
		    } else if ($identifier == self::$IMAGEID) {
			    if ($value == 0) {
				    $value = null;
			    }
			    $this->prodimageid = $value;
		    } else if ($identifier == self::$DISPLAY) {
			    $value = strtoupper($value);
			    if ($value == 'KG') {
				    $value = null;
			    } else if (($value != 'K') && ($value != 'G')) {
				    $value = null;
			    }
			    $this->display = $value;
		    }
		}
	    }
	    
	    if (is_null($this->priceB)) {
		$this->priceB = $this->priceA;
	    }
	    if (is_null($this->priceC)) {
		$this->priceC = $this->priceA;
	    }
	    if (is_null($this->shortName)) {
		$this->shortName = $this->longName;
	    }
	    return array("status" => "OK");
	} catch (Exception $e) {
	    return array("status" => "ERROR","code" => PARSE_ERROR,"msg" => PARSE_ERROR_MSG,"line" => $aTextLine);
	}
    }
	
    function getShortName() {
	return $this->shortName;
    }

    function getPriceA() {
	return $this->priceA;
    }

    function getPriceB() {
	return $this->priceB;
    }

    function getPriceC() {
	return $this->priceC;
    }
    
    function getBarcode() {
	    return $this->barcode;
    }
    
    function getUnit() {
	    return $this->unit;
    }

    function getDays() {
	    return $this->days;
    }
    function getTax() {
	return $this->tax;
    }
    function getTogoTax() {
	return $this->togotax;
    }
    function getLongName() {
	return $this->longName;
    }

    function getProdId() {
	return $this->prodid;
    }

    function getAvailable() {
	return $this->available;
    }
    
    function getTaxAustria() {
	return $this->taxaustria;
    }
    function getAmount() {
	return $this->amount;
    }

    function getCategory() {
	return $this->category;
    }
    function getAudio() {
	return $this->audio;
    }
    function getFavorite() {
	return $this->favorite;
    }
    function getProdImageId() {
	    return (($this->prodimageid == 0) ? null : $this->prodimageid);
    }
    function getDisplay() {
	    return (($this->display == 'KG') ? null : $this->display);
    }
    function toString() {
	    return "S:" . $this->shortName . " PA:" . $this->priceA . " PB:" . $this->priceB . " PC:" . $this->priceC  . " Barcode:" . $this->barcode . " Tax:" . $this->tax  . "Togotax:" . $this->togotax . " R:" . $this->longName;
    }
}